@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2018 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,11 +20,13 @@
 */
 #include "../SDL_internal.h"
 
-#ifndef _SDL_dropevents_c_h
-#define _SDL_dropevents_c_h
+#ifndef SDL_dropevents_c_h_
+#define SDL_dropevents_c_h_
 
-extern int SDL_SendDropFile(const char *file);
+extern int SDL_SendDropFile(SDL_Window *window, const char *file);
+extern int SDL_SendDropText(SDL_Window *window, const char *text);
+extern int SDL_SendDropComplete(SDL_Window *window);
 
-#endif /* _SDL_dropevents_c_h */
+#endif /* SDL_dropevents_c_h_ */
 
 /* vi: set ts=4 sw=4 expandtab: */
