@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2018 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,12 +20,12 @@
 */
 #include "../SDL_internal.h"
 
-#ifndef _SDL_windowevents_c_h
-#define _SDL_windowevents_c_h
+#ifndef SDL_windowevents_c_h_
+#define SDL_windowevents_c_h_
 
 extern int SDL_SendWindowEvent(SDL_Window * window, Uint8 windowevent,
                                int data1, int data2);
 
-#endif /* _SDL_windowevents_c_h */
+#endif /* SDL_windowevents_c_h_ */
 
 /* vi: set ts=4 sw=4 expandtab: */
