@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2011 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2018 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,15 +18,18 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "SDL_config.h"
+#include "../../SDL_internal.h"
 
 #ifdef SDL_HAPTIC_IOKIT
 
+#include "SDL_assert.h"
+#include "SDL_stdinc.h"
 #include "SDL_haptic.h"
 #include "../SDL_syshaptic.h"
 #include "SDL_joystick.h"
 #include "../../joystick/SDL_sysjoystick.h"     /* For the real SDL_Joystick */
 #include "../../joystick/darwin/SDL_sysjoystick_c.h"    /* For joystick hwdata */
+#include "SDL_syshaptic_c.h"
 
 #include <IOKit/IOKitLib.h>
 #include <IOKit/hid/IOHIDKeys.h>
@@ -35,26 +38,25 @@
 #include <ForceFeedback/ForceFeedbackConstants.h>
 
 #ifndef IO_OBJECT_NULL
-#define IO_OBJECT_NULL	((io_service_t)0)
+#define IO_OBJECT_NULL  ((io_service_t)0)
 #endif
 
-#define MAX_HAPTICS  32
-
-
 /*
  * List of available haptic devices.
  */
-static struct
+typedef struct SDL_hapticlist_item
 {
     char name[256];             /* Name of the device. */
 
     io_service_t dev;           /* Node we use to create the device. */
-    SDL_Haptic *haptic;         /* Haptic currently assosciated with it. */
+    SDL_Haptic *haptic;         /* Haptic currently associated with it. */
 
     /* Usage pages for determining if it's a mouse or not. */
     long usage;
     long usagePage;
-} SDL_hapticlist[MAX_HAPTICS];
+
+    struct SDL_hapticlist_item *next;
+} SDL_hapticlist_item;
 
 
 /*
@@ -82,19 +84,22 @@ struct haptic_hweffect
 static void SDL_SYS_HapticFreeFFEFFECT(FFEFFECT * effect, int type);
 static int HIDGetDeviceProduct(io_service_t dev, char *name);
 
+static SDL_hapticlist_item *SDL_hapticlist = NULL;
+static SDL_hapticlist_item *SDL_hapticlist_tail = NULL;
+static int numhaptics = -1;
 
-/* 
+/*
  * Like strerror but for force feedback errors.
  */
 static const char *
-FFStrError(HRESULT err)
+FFStrError(unsigned int err)
 {
     switch (err) {
     case FFERR_DEVICEFULL:
         return "device full";
-        /* This should be valid, but for some reason isn't defined... */
-        /*case FFERR_DEVICENOTREG:
-           return "device not registered"; */
+    /* This should be valid, but for some reason isn't defined... */
+    /* case FFERR_DEVICENOTREG:
+        return "device not registered"; */
     case FFERR_DEVICEPAUSED:
         return "device paused";
     case FFERR_DEVICERELEASED:
@@ -146,93 +151,181 @@ FFStrError(HRESULT err)
 int
 SDL_SYS_HapticInit(void)
 {
-    int numhaptics;
     IOReturn result;
     io_iterator_t iter;
     CFDictionaryRef match;
     io_service_t device;
-    CFMutableDictionaryRef hidProperties;
-    CFTypeRef refCF;
 
-    /* Clear all the memory. */
-    SDL_memset(SDL_hapticlist, 0, sizeof(SDL_hapticlist));
+    if (numhaptics != -1) {
+        return SDL_SetError("Haptic subsystem already initialized!");
+    }
+    numhaptics = 0;
 
     /* Get HID devices. */
     match = IOServiceMatching(kIOHIDDeviceKey);
     if (match == NULL) {
-        SDL_SetError("Haptic: Failed to get IOServiceMatching.");
-        return -1;
+        return SDL_SetError("Haptic: Failed to get IOServiceMatching.");
     }
 
     /* Now search I/O Registry for matching devices. */
     result = IOServiceGetMatchingServices(kIOMasterPortDefault, match, &iter);
     if (result != kIOReturnSuccess) {
-        SDL_SetError("Haptic: Couldn't create a HID object iterator.");
-        return -1;
+        return SDL_SetError("Haptic: Couldn't create a HID object iterator.");
     }
     /* IOServiceGetMatchingServices consumes dictionary. */
 
     if (!IOIteratorIsValid(iter)) {     /* No iterator. */
-        numhaptics = 0;
         return 0;
     }
 
-    numhaptics = 0;
     while ((device = IOIteratorNext(iter)) != IO_OBJECT_NULL) {
-
-        /* Check for force feedback. */
-        if (FFIsForceFeedback(device) == FF_OK) {
-
-            /* Set basic device data. */
-            HIDGetDeviceProduct(device, SDL_hapticlist[numhaptics].name);
-            SDL_hapticlist[numhaptics].dev = device;
-            SDL_hapticlist[numhaptics].haptic = NULL;
-
-            /* Set usage pages. */
-            hidProperties = 0;
-            refCF = 0;
-            result = IORegistryEntryCreateCFProperties(device,
-                                                       &hidProperties,
-                                                       kCFAllocatorDefault,
-                                                       kNilOptions);
-            if ((result == KERN_SUCCESS) && hidProperties) {
-                refCF =
-                    CFDictionaryGetValue(hidProperties,
-                                         CFSTR(kIOHIDPrimaryUsagePageKey));
-                if (refCF) {
-                    if (!CFNumberGetValue(refCF, kCFNumberLongType,
-                                          &SDL_hapticlist[numhaptics].
-                                          usagePage))
-                        SDL_SetError
-                            ("Haptic: Recieving device's usage page.");
-                    refCF =
-                        CFDictionaryGetValue(hidProperties,
-                                             CFSTR(kIOHIDPrimaryUsageKey));
-                    if (refCF) {
-                        if (!CFNumberGetValue(refCF, kCFNumberLongType,
-                                              &SDL_hapticlist[numhaptics].
-                                              usage))
-                            SDL_SetError("Haptic: Recieving device's usage.");
-                    }
-                }
-                CFRelease(hidProperties);
-            }
-
-            /* Device has been added. */
-            numhaptics++;
-        } else {                /* Free the unused device. */
-            IOObjectRelease(device);
-        }
-
-        /* Reached haptic limit. */
-        if (numhaptics >= MAX_HAPTICS)
-            break;
+        MacHaptic_MaybeAddDevice(device);
+        /* always release as the AddDevice will retain IF it's a forcefeedback device */
+        IOObjectRelease(device);
     }
     IOObjectRelease(iter);
 
     return numhaptics;
 }
 
+int
+SDL_SYS_NumHaptics(void)
+{
+    return numhaptics;
+}
+
+static SDL_hapticlist_item *
+HapticByDevIndex(int device_index)
+{
+    SDL_hapticlist_item *item = SDL_hapticlist;
+
+    if ((device_index < 0) || (device_index >= numhaptics)) {
+        return NULL;
+    }
+
+    while (device_index > 0) {
+        SDL_assert(item != NULL);
+        --device_index;
+        item = item->next;
+    }
+
+    return item;
+}
+
+int
+MacHaptic_MaybeAddDevice( io_object_t device )
+{
+    IOReturn result;
+    CFMutableDictionaryRef hidProperties;
+    CFTypeRef refCF;
+    SDL_hapticlist_item *item;
+
+    if (numhaptics == -1) {
+        return -1; /* not initialized. We'll pick these up on enumeration if we init later. */
+    }
+
+    /* Check for force feedback. */
+    if (FFIsForceFeedback(device) != FF_OK) {
+        return -1;
+    }
+
+    /* Make sure we don't already have it */
+    for (item = SDL_hapticlist; item ; item = item->next)
+    {
+        if (IOObjectIsEqualTo((io_object_t) item->dev, device)) {
+            /* Already added */
+            return -1;
+        }
+    }
+
+    item = (SDL_hapticlist_item *)SDL_calloc(1, sizeof(SDL_hapticlist_item));
+    if (item == NULL) {
+        return SDL_SetError("Could not allocate haptic storage");
+    }
+
+    /* retain it as we are going to keep it around a while */
+    IOObjectRetain(device);
+
+    /* Set basic device data. */
+    HIDGetDeviceProduct(device, item->name);
+    item->dev = device;
+
+    /* Set usage pages. */
+    hidProperties = 0;
+    refCF = 0;
+    result = IORegistryEntryCreateCFProperties(device,
+                                               &hidProperties,
+                                               kCFAllocatorDefault,
+                                               kNilOptions);
+    if ((result == KERN_SUCCESS) && hidProperties) {
+        refCF = CFDictionaryGetValue(hidProperties,
+                                     CFSTR(kIOHIDPrimaryUsagePageKey));
+        if (refCF) {
+            if (!CFNumberGetValue(refCF, kCFNumberLongType, &item->usagePage)) {
+                SDL_SetError("Haptic: Receiving device's usage page.");
+            }
+            refCF = CFDictionaryGetValue(hidProperties,
+                                         CFSTR(kIOHIDPrimaryUsageKey));
+            if (refCF) {
+                if (!CFNumberGetValue(refCF, kCFNumberLongType, &item->usage)) {
+                    SDL_SetError("Haptic: Receiving device's usage.");
+                }
+            }
+        }
+        CFRelease(hidProperties);
+    }
+
+    if (SDL_hapticlist_tail == NULL) {
+        SDL_hapticlist = SDL_hapticlist_tail = item;
+    } else {
+        SDL_hapticlist_tail->next = item;
+        SDL_hapticlist_tail = item;
+    }
+
+    /* Device has been added. */
+    ++numhaptics;
+
+    return numhaptics;
+}
+
+int
+MacHaptic_MaybeRemoveDevice( io_object_t device )
+{
+    SDL_hapticlist_item *item;
+    SDL_hapticlist_item *prev = NULL;
+
+    if (numhaptics == -1) {
+        return -1; /* not initialized. ignore this. */
+    }
+
+    for (item = SDL_hapticlist; item != NULL; item = item->next) {
+        /* found it, remove it. */
+        if (IOObjectIsEqualTo((io_object_t) item->dev, device)) {
+            const int retval = item->haptic ? item->haptic->index : -1;
+
+            if (prev != NULL) {
+                prev->next = item->next;
+            } else {
+                SDL_assert(SDL_hapticlist == item);
+                SDL_hapticlist = item->next;
+            }
+            if (item == SDL_hapticlist_tail) {
+                SDL_hapticlist_tail = prev;
+            }
+
+            /* Need to decrement the haptic count */
+            --numhaptics;
+            /* !!! TODO: Send a haptic remove event? */
+
+            IOObjectRelease(item->dev);
+            SDL_free(item);
+            return retval;
+        }
+        prev = item;
+    }
+
+    return -1;
+}
 
 /*
  * Return the name of a haptic device, does not need to be opened.
@@ -240,7 +333,9 @@ SDL_SYS_HapticInit(void)
 const char *
 SDL_SYS_HapticName(int index)
 {
-    return SDL_hapticlist[index].name;
+    SDL_hapticlist_item *item;
+    item = HapticByDevIndex(index);
+    return item->name;
 }
 
 /*
@@ -258,12 +353,11 @@ HIDGetDeviceProduct(io_service_t dev, char *name)
     ret = IORegistryEntryCreateCFProperties(dev, &hidProperties,
                                             kCFAllocatorDefault, kNilOptions);
     if ((ret != KERN_SUCCESS) || !hidProperties) {
-        SDL_SetError("Haptic: Unable to create CFProperties.");
-        return -1;
+        return SDL_SetError("Haptic: Unable to create CFProperties.");
     }
 
     /* Mac OS X currently is not mirroring all USB properties to HID page so need to look at USB device page also
-     * get dictionary for usb properties: step up two levels and get CF dictionary for USB properties
+     * get dictionary for USB properties: step up two levels and get CF dictionary for USB properties
      */
     if ((KERN_SUCCESS ==
          IORegistryEntryGetParentEntry(dev, kIOServicePlane, &parent1))
@@ -276,31 +370,26 @@ HIDGetDeviceProduct(io_service_t dev, char *name)
         if (usbProperties) {
             CFTypeRef refCF = 0;
             /* get device info
-             * try hid dictionary first, if fail then go to usb dictionary
+             * try hid dictionary first, if fail then go to USB dictionary
              */
 
 
             /* Get product name */
-            refCF =
-                CFDictionaryGetValue(hidProperties, CFSTR(kIOHIDProductKey));
-            if (!refCF)
-                refCF =
-                    CFDictionaryGetValue(usbProperties,
-                                         CFSTR("USB Product Name"));
+            refCF = CFDictionaryGetValue(hidProperties, CFSTR(kIOHIDProductKey));
+            if (!refCF) {
+                refCF = CFDictionaryGetValue(usbProperties,
+                                             CFSTR("USB Product Name"));
+            }
             if (refCF) {
                 if (!CFStringGetCString(refCF, name, 256,
                                         CFStringGetSystemEncoding())) {
-                    SDL_SetError
-                        ("Haptic: CFStringGetCString error retrieving pDevice->product.");
-                    return -1;
+                    return SDL_SetError("Haptic: CFStringGetCString error retrieving pDevice->product.");
                 }
             }
 
             CFRelease(usbProperties);
         } else {
-            SDL_SetError
-                ("Haptic: IORegistryEntryCreateCFProperties failed to create usbProperties.");
-            return -1;
+            return SDL_SetError("Haptic: IORegistryEntryCreateCFProperties failed to create usbProperties.");
         }
 
         /* Release stuff. */
@@ -311,8 +400,7 @@ HIDGetDeviceProduct(io_service_t dev, char *name)
             SDL_SetError("Haptic: IOObjectRelease error with parent1.");
         }
     } else {
-        SDL_SetError("Haptic: Error getting registry entries.");
-        return -1;
+        return SDL_SetError("Haptic: Error getting registry entries.");
     }
 
     return 0;
@@ -337,8 +425,7 @@ GetSupportedFeatures(SDL_Haptic * haptic)
 
     ret = FFDeviceGetForceFeedbackCapabilities(device, &features);
     if (ret != FF_OK) {
-        SDL_SetError("Haptic: Unable to get device's supported features.");
-        return -1;
+        return SDL_SetError("Haptic: Unable to get device's supported features.");
     }
 
     supported = 0;
@@ -350,7 +437,8 @@ GetSupportedFeatures(SDL_Haptic * haptic)
     /* Test for effects. */
     FF_TEST(FFCAP_ET_CONSTANTFORCE, SDL_HAPTIC_CONSTANT);
     FF_TEST(FFCAP_ET_RAMPFORCE, SDL_HAPTIC_RAMP);
-    FF_TEST(FFCAP_ET_SQUARE, SDL_HAPTIC_SQUARE);
+    /* !!! FIXME: put this back when we have more bits in 2.1 */
+    /* FF_TEST(FFCAP_ET_SQUARE, SDL_HAPTIC_SQUARE); */
     FF_TEST(FFCAP_ET_SINE, SDL_HAPTIC_SINE);
     FF_TEST(FFCAP_ET_TRIANGLE, SDL_HAPTIC_TRIANGLE);
     FF_TEST(FFCAP_ET_SAWTOOTHUP, SDL_HAPTIC_SAWTOOTHUP);
@@ -364,24 +452,22 @@ GetSupportedFeatures(SDL_Haptic * haptic)
     /* Check if supports gain. */
     ret = FFDeviceGetForceFeedbackProperty(device, FFPROP_FFGAIN,
                                            &val, sizeof(val));
-    if (ret == FF_OK)
+    if (ret == FF_OK) {
         supported |= SDL_HAPTIC_GAIN;
-    else if (ret != FFERR_UNSUPPORTED) {
-        SDL_SetError("Haptic: Unable to get if device supports gain: %s.",
-                     FFStrError(ret));
-        return -1;
+    } else if (ret != FFERR_UNSUPPORTED) {
+        return SDL_SetError("Haptic: Unable to get if device supports gain: %s.",
+                            FFStrError(ret));
     }
 
     /* Checks if supports autocenter. */
     ret = FFDeviceGetForceFeedbackProperty(device, FFPROP_AUTOCENTER,
                                            &val, sizeof(val));
-    if (ret == FF_OK)
+    if (ret == FF_OK) {
         supported |= SDL_HAPTIC_AUTOCENTER;
-    else if (ret != FFERR_UNSUPPORTED) {
-        SDL_SetError
+    } else if (ret != FFERR_UNSUPPORTED) {
+        return SDL_SetError
             ("Haptic: Unable to get if device supports autocenter: %s.",
              FFStrError(ret));
-        return -1;
     }
 
     /* Check for axes, we have an artificial limit on axes */
@@ -394,7 +480,7 @@ GetSupportedFeatures(SDL_Haptic * haptic)
     supported |= SDL_HAPTIC_STATUS | SDL_HAPTIC_PAUSE;
 
     haptic->supported = supported;
-    return 0;;
+    return 0;
 }
 
 
@@ -426,7 +512,7 @@ SDL_SYS_HapticOpenFromService(SDL_Haptic * haptic, io_service_t service)
 
     /* Get supported features. */
     ret2 = GetSupportedFeatures(haptic);
-    if (haptic->supported < 0) {
+    if (ret2 < 0) {
         goto open_err;
     }
 
@@ -465,7 +551,7 @@ SDL_SYS_HapticOpenFromService(SDL_Haptic * haptic, io_service_t service)
     FFReleaseDevice(haptic->hwdata->device);
   creat_err:
     if (haptic->hwdata != NULL) {
-        free(haptic->hwdata);
+        SDL_free(haptic->hwdata);
         haptic->hwdata = NULL;
     }
     return -1;
@@ -479,8 +565,10 @@ SDL_SYS_HapticOpenFromService(SDL_Haptic * haptic, io_service_t service)
 int
 SDL_SYS_HapticOpen(SDL_Haptic * haptic)
 {
-    return SDL_SYS_HapticOpenFromService(haptic,
-                                         SDL_hapticlist[haptic->index].dev);
+    SDL_hapticlist_item *item;
+    item = HapticByDevIndex(haptic->index);
+
+    return SDL_SYS_HapticOpenFromService(haptic, item->dev);
 }
 
 
@@ -490,12 +578,15 @@ SDL_SYS_HapticOpen(SDL_Haptic * haptic)
 int
 SDL_SYS_HapticMouse(void)
 {
-    int i;
+    int device_index = 0;
+    SDL_hapticlist_item *item;
 
-    for (i = 0; i < SDL_numhaptics; i++) {
-        if ((SDL_hapticlist[i].usagePage == kHIDPage_GenericDesktop) &&
-            (SDL_hapticlist[i].usage == kHIDUsage_GD_Mouse))
-            return i;
+    for (item = SDL_hapticlist; item; item = item->next) {
+        if ((item->usagePage == kHIDPage_GenericDesktop) &&
+            (item->usage == kHIDUsage_GD_Mouse)) {
+            return device_index;
+        }
+        ++device_index;
     }
 
     return -1;
@@ -508,21 +599,23 @@ SDL_SYS_HapticMouse(void)
 int
 SDL_SYS_JoystickIsHaptic(SDL_Joystick * joystick)
 {
-    if (joystick->hwdata->ffservice != 0)
+    if (joystick->hwdata->ffservice != 0) {
         return SDL_TRUE;
+    }
     return SDL_FALSE;
 }
 
 
 /*
- * Checks to see if the haptic device and joystick and in reality the same.
+ * Checks to see if the haptic device and joystick are in reality the same.
  */
 int
 SDL_SYS_JoystickSameHaptic(SDL_Haptic * haptic, SDL_Joystick * joystick)
 {
     if (IOObjectIsEqualTo((io_object_t) ((size_t)haptic->hwdata->device),
-                          joystick->hwdata->ffservice))
+                          joystick->hwdata->ffservice)) {
         return 1;
+    }
     return 0;
 }
 
@@ -533,16 +626,16 @@ SDL_SYS_JoystickSameHaptic(SDL_Haptic * haptic, SDL_Joystick * joystick)
 int
 SDL_SYS_HapticOpenFromJoystick(SDL_Haptic * haptic, SDL_Joystick * joystick)
 {
-    int i;
-    for (i=0; i<SDL_numhaptics; i++) {
-       if (IOObjectIsEqualTo((io_object_t) SDL_hapticlist[i].dev,
+    int device_index = 0;
+    SDL_hapticlist_item *item;
+
+    for (item = SDL_hapticlist; item; item = item->next) {
+        if (IOObjectIsEqualTo((io_object_t) item->dev,
                              joystick->hwdata->ffservice)) {
-           haptic->index = i;
+           haptic->index = device_index;
            break;
-       }
-    }
-    if (i >= SDL_numhaptics) {
-       return -1;
+        }
+        ++device_index;
     }
 
     return SDL_SYS_HapticOpenFromService(haptic, joystick->hwdata->ffservice);
@@ -572,21 +665,28 @@ SDL_SYS_HapticClose(SDL_Haptic * haptic)
 }
 
 
-/* 
+/*
  * Clean up after system specific haptic stuff
  */
 void
 SDL_SYS_HapticQuit(void)
 {
-    int i;
+    SDL_hapticlist_item *item;
+    SDL_hapticlist_item *next = NULL;
 
-    for (i = 0; i < SDL_numhaptics; i++) {
+    for (item = SDL_hapticlist; item; item = next) {
+        next = item->next;
         /* Opened and not closed haptics are leaked, this is on purpose.
          * Close your haptic devices after usage. */
 
         /* Free the io_service_t */
-        IOObjectRelease(SDL_hapticlist[i].dev);
+        IOObjectRelease(item->dev);
+        SDL_free(item);
     }
+
+    numhaptics = -1;
+    SDL_hapticlist = NULL;
+    SDL_hapticlist_tail = NULL;
 }
 
 
@@ -626,8 +726,7 @@ SDL_SYS_SetDirection(FFEFFECT * effect, SDL_HapticDirection * dir, int naxes)
     /* Has axes. */
     rglDir = SDL_malloc(sizeof(LONG) * naxes);
     if (rglDir == NULL) {
-        SDL_OutOfMemory();
-        return -1;
+        return SDL_OutOfMemory();
     }
     SDL_memset(rglDir, 0, sizeof(LONG) * naxes);
     effect->rglDirection = rglDir;
@@ -640,23 +739,26 @@ SDL_SYS_SetDirection(FFEFFECT * effect, SDL_HapticDirection * dir, int naxes)
     case SDL_HAPTIC_CARTESIAN:
         effect->dwFlags |= FFEFF_CARTESIAN;
         rglDir[0] = dir->dir[0];
-        if (naxes > 1)
+        if (naxes > 1) {
             rglDir[1] = dir->dir[1];
-        if (naxes > 2)
+        }
+        if (naxes > 2) {
             rglDir[2] = dir->dir[2];
+        }
         return 0;
     case SDL_HAPTIC_SPHERICAL:
         effect->dwFlags |= FFEFF_SPHERICAL;
         rglDir[0] = dir->dir[0];
-        if (naxes > 1)
+        if (naxes > 1) {
             rglDir[1] = dir->dir[1];
-        if (naxes > 2)
+        }
+        if (naxes > 2) {
             rglDir[2] = dir->dir[2];
+        }
         return 0;
 
     default:
-        SDL_SetError("Haptic: Unknown direction type.");
-        return -1;
+        return SDL_SetError("Haptic: Unknown direction type.");
     }
 }
 
@@ -669,22 +771,21 @@ SDL_SYS_SetDirection(FFEFFECT * effect, SDL_HapticDirection * dir, int naxes)
  * Creates the FFEFFECT from a SDL_HapticEffect.
  */
 static int
-SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest,
-                   SDL_HapticEffect * src)
+SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest, SDL_HapticEffect * src)
 {
     int i;
-    FFCONSTANTFORCE *constant;
-    FFPERIODIC *periodic;
-    FFCONDITION *condition;     /* Actually an array of conditions - one per axis. */
-    FFRAMPFORCE *ramp;
-    FFCUSTOMFORCE *custom;
-    FFENVELOPE *envelope;
-    SDL_HapticConstant *hap_constant;
-    SDL_HapticPeriodic *hap_periodic;
-    SDL_HapticCondition *hap_condition;
-    SDL_HapticRamp *hap_ramp;
-    SDL_HapticCustom *hap_custom;
-    DWORD *axes;
+    FFCONSTANTFORCE *constant = NULL;
+    FFPERIODIC *periodic = NULL;
+    FFCONDITION *condition = NULL;     /* Actually an array of conditions - one per axis. */
+    FFRAMPFORCE *ramp = NULL;
+    FFCUSTOMFORCE *custom = NULL;
+    FFENVELOPE *envelope = NULL;
+    SDL_HapticConstant *hap_constant = NULL;
+    SDL_HapticPeriodic *hap_periodic = NULL;
+    SDL_HapticCondition *hap_condition = NULL;
+    SDL_HapticRamp *hap_ramp = NULL;
+    SDL_HapticCustom *hap_custom = NULL;
+    DWORD *axes = NULL;
 
     /* Set global stuff. */
     SDL_memset(dest, 0, sizeof(FFEFFECT));
@@ -696,8 +797,7 @@ SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest,
     /* Envelope. */
     envelope = SDL_malloc(sizeof(FFENVELOPE));
     if (envelope == NULL) {
-        SDL_OutOfMemory();
-        return -1;
+        return SDL_OutOfMemory();
     }
     SDL_memset(envelope, 0, sizeof(FFENVELOPE));
     dest->lpEnvelope = envelope;
@@ -708,8 +808,7 @@ SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest,
     if (dest->cAxes > 0) {
         axes = SDL_malloc(sizeof(DWORD) * dest->cAxes);
         if (axes == NULL) {
-            SDL_OutOfMemory();
-            return -1;
+            return SDL_OutOfMemory();
         }
         axes[0] = haptic->hwdata->axes[0];      /* Always at least one axis. */
         if (dest->cAxes > 1) {
@@ -722,14 +821,13 @@ SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest,
     }
 
 
-    /* The big type handling switch, even bigger then linux's version. */
+    /* The big type handling switch, even bigger then Linux's version. */
     switch (src->type) {
     case SDL_HAPTIC_CONSTANT:
         hap_constant = &src->constant;
         constant = SDL_malloc(sizeof(FFCONSTANTFORCE));
         if (constant == NULL) {
-            SDL_OutOfMemory();
-            return -1;
+            return SDL_OutOfMemory();
         }
         SDL_memset(constant, 0, sizeof(FFCONSTANTFORCE));
 
@@ -765,22 +863,23 @@ SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest,
         break;
 
     case SDL_HAPTIC_SINE:
-    case SDL_HAPTIC_SQUARE:
+    /* !!! FIXME: put this back when we have more bits in 2.1 */
+    /* case SDL_HAPTIC_SQUARE: */
     case SDL_HAPTIC_TRIANGLE:
     case SDL_HAPTIC_SAWTOOTHUP:
     case SDL_HAPTIC_SAWTOOTHDOWN:
         hap_periodic = &src->periodic;
         periodic = SDL_malloc(sizeof(FFPERIODIC));
         if (periodic == NULL) {
-            SDL_OutOfMemory();
-            return -1;
+            return SDL_OutOfMemory();
         }
         SDL_memset(periodic, 0, sizeof(FFPERIODIC));
 
         /* Specifics */
-        periodic->dwMagnitude = CONVERT(hap_periodic->magnitude);
+        periodic->dwMagnitude = CONVERT(SDL_abs(hap_periodic->magnitude));
         periodic->lOffset = CONVERT(hap_periodic->offset);
-        periodic->dwPhase = hap_periodic->phase;
+        periodic->dwPhase = 
+                (hap_periodic->phase + (hap_periodic->magnitude < 0 ? 18000 : 0)) % 36000;
         periodic->dwPeriod = hap_periodic->period * 1000;
         dest->cbTypeSpecificParams = sizeof(FFPERIODIC);
         dest->lpvTypeSpecificParams = periodic;
@@ -816,26 +915,28 @@ SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest,
     case SDL_HAPTIC_INERTIA:
     case SDL_HAPTIC_FRICTION:
         hap_condition = &src->condition;
-        condition = SDL_malloc(sizeof(FFCONDITION) * dest->cAxes);
-        if (condition == NULL) {
-            SDL_OutOfMemory();
-            return -1;
-        }
-        SDL_memset(condition, 0, sizeof(FFCONDITION));
+        if (dest->cAxes > 0) {
+            condition = SDL_malloc(sizeof(FFCONDITION) * dest->cAxes);
+            if (condition == NULL) {
+                return SDL_OutOfMemory();
+            }
+            SDL_memset(condition, 0, sizeof(FFCONDITION));
 
-        /* Specifics */
-        for (i = 0; i < dest->cAxes; i++) {
-            condition[i].lOffset = CONVERT(hap_condition->center[i]);
-            condition[i].lPositiveCoefficient =
-                CONVERT(hap_condition->right_coeff[i]);
-            condition[i].lNegativeCoefficient =
-                CONVERT(hap_condition->left_coeff[i]);
-            condition[i].dwPositiveSaturation =
-                CCONVERT(hap_condition->right_sat[i]);
-            condition[i].dwNegativeSaturation =
-                CCONVERT(hap_condition->left_sat[i]);
-            condition[i].lDeadBand = CCONVERT(hap_condition->deadband[i]);
+            /* Specifics */
+            for (i = 0; i < dest->cAxes; i++) {
+                condition[i].lOffset = CONVERT(hap_condition->center[i]);
+                condition[i].lPositiveCoefficient =
+                    CONVERT(hap_condition->right_coeff[i]);
+                condition[i].lNegativeCoefficient =
+                    CONVERT(hap_condition->left_coeff[i]);
+                condition[i].dwPositiveSaturation =
+                    CCONVERT(hap_condition->right_sat[i] / 2);
+                condition[i].dwNegativeSaturation =
+                    CCONVERT(hap_condition->left_sat[i] / 2);
+                condition[i].lDeadBand = CCONVERT(hap_condition->deadband[i] / 2);
+            }
         }
+
         dest->cbTypeSpecificParams = sizeof(FFCONDITION) * dest->cAxes;
         dest->lpvTypeSpecificParams = condition;
 
@@ -861,8 +962,7 @@ SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest,
         hap_ramp = &src->ramp;
         ramp = SDL_malloc(sizeof(FFRAMPFORCE));
         if (ramp == NULL) {
-            SDL_OutOfMemory();
-            return -1;
+            return SDL_OutOfMemory();
         }
         SDL_memset(ramp, 0, sizeof(FFRAMPFORCE));
 
@@ -900,8 +1000,7 @@ SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest,
         hap_custom = &src->custom;
         custom = SDL_malloc(sizeof(FFCUSTOMFORCE));
         if (custom == NULL) {
-            SDL_OutOfMemory();
-            return -1;
+            return SDL_OutOfMemory();
         }
         SDL_memset(custom, 0, sizeof(FFCUSTOMFORCE));
 
@@ -945,8 +1044,7 @@ SDL_SYS_ToFFEFFECT(SDL_Haptic * haptic, FFEFFECT * dest,
 
 
     default:
-        SDL_SetError("Haptic: Unknown effect type.");
-        return -1;
+        return SDL_SetError("Haptic: Unknown effect type.");
     }
 
     return 0;
@@ -961,14 +1059,10 @@ SDL_SYS_HapticFreeFFEFFECT(FFEFFECT * effect, int type)
 {
     FFCUSTOMFORCE *custom;
 
-    if (effect->lpEnvelope != NULL) {
-        SDL_free(effect->lpEnvelope);
-        effect->lpEnvelope = NULL;
-    }
-    if (effect->rgdwAxes != NULL) {
-        SDL_free(effect->rgdwAxes);
-        effect->rgdwAxes = NULL;
-    }
+    SDL_free(effect->lpEnvelope);
+    effect->lpEnvelope = NULL;
+    SDL_free(effect->rgdwAxes);
+    effect->rgdwAxes = NULL;
     if (effect->lpvTypeSpecificParams != NULL) {
         if (type == SDL_HAPTIC_CUSTOM) {        /* Must free the custom data. */
             custom = (FFCUSTOMFORCE *) effect->lpvTypeSpecificParams;
@@ -978,10 +1072,8 @@ SDL_SYS_HapticFreeFFEFFECT(FFEFFECT * effect, int type)
         SDL_free(effect->lpvTypeSpecificParams);
         effect->lpvTypeSpecificParams = NULL;
     }
-    if (effect->rglDirection != NULL) {
-        SDL_free(effect->rglDirection);
-        effect->rglDirection = NULL;
-    }
+    SDL_free(effect->rglDirection);
+    effect->rglDirection = NULL;
 }
 
 
@@ -998,8 +1090,9 @@ SDL_SYS_HapticEffectType(Uint16 type)
     case SDL_HAPTIC_RAMP:
         return kFFEffectType_RampForce_ID;
 
-    case SDL_HAPTIC_SQUARE:
-        return kFFEffectType_Square_ID;
+    /* !!! FIXME: put this back when we have more bits in 2.1 */
+    /* case SDL_HAPTIC_SQUARE:
+        return kFFEffectType_Square_ID; */
 
     case SDL_HAPTIC_SINE:
         return kFFEffectType_Sine_ID;
@@ -1078,10 +1171,8 @@ SDL_SYS_HapticNewEffect(SDL_Haptic * haptic, struct haptic_effect *effect,
   err_effectdone:
     SDL_SYS_HapticFreeFFEFFECT(&effect->hweffect->effect, base->type);
   err_hweffect:
-    if (effect->hweffect != NULL) {
-        SDL_free(effect->hweffect);
-        effect->hweffect = NULL;
-    }
+    SDL_free(effect->hweffect);
+    effect->hweffect = NULL;
     return -1;
 }
 
@@ -1151,9 +1242,8 @@ SDL_SYS_HapticRunEffect(SDL_Haptic * haptic, struct haptic_effect *effect,
     /* Run the effect. */
     ret = FFEffectStart(effect->hweffect->ref, iter, 0);
     if (ret != FF_OK) {
-        SDL_SetError("Haptic: Unable to run the effect: %s.",
-                     FFStrError(ret));
-        return -1;
+        return SDL_SetError("Haptic: Unable to run the effect: %s.",
+                            FFStrError(ret));
     }
 
     return 0;
@@ -1170,9 +1260,8 @@ SDL_SYS_HapticStopEffect(SDL_Haptic * haptic, struct haptic_effect *effect)
 
     ret = FFEffectStop(effect->hweffect->ref);
     if (ret != FF_OK) {
-        SDL_SetError("Haptic: Unable to stop the effect: %s.",
-                     FFStrError(ret));
-        return -1;
+        return SDL_SetError("Haptic: Unable to stop the effect: %s.",
+                            FFStrError(ret));
     }
 
     return 0;
@@ -1187,8 +1276,7 @@ SDL_SYS_HapticDestroyEffect(SDL_Haptic * haptic, struct haptic_effect *effect)
 {
     HRESULT ret;
 
-    ret =
-        FFDeviceReleaseEffect(haptic->hwdata->device, effect->hweffect->ref);
+    ret = FFDeviceReleaseEffect(haptic->hwdata->device, effect->hweffect->ref);
     if (ret != FF_OK) {
         SDL_SetError("Haptic: Error removing the effect from the device: %s.",
                      FFStrError(ret));
@@ -1217,8 +1305,9 @@ SDL_SYS_HapticGetEffectStatus(SDL_Haptic * haptic,
         return -1;
     }
 
-    if (status == 0)
+    if (status == 0) {
         return SDL_FALSE;
+    }
     return SDL_TRUE;            /* Assume it's playing or emulated. */
 }
 
@@ -1233,12 +1322,10 @@ SDL_SYS_HapticSetGain(SDL_Haptic * haptic, int gain)
     Uint32 val;
 
     val = gain * 100;           /* Mac OS X uses 0 to 10,000 */
-    ret =
-        FFDeviceSetForceFeedbackProperty(haptic->hwdata->device,
-                                         FFPROP_FFGAIN, &val);
+    ret = FFDeviceSetForceFeedbackProperty(haptic->hwdata->device,
+                                           FFPROP_FFGAIN, &val);
     if (ret != FF_OK) {
-        SDL_SetError("Haptic: Error setting gain: %s.", FFStrError(ret));
-        return -1;
+        return SDL_SetError("Haptic: Error setting gain: %s.", FFStrError(ret));
     }
 
     return 0;
@@ -1255,17 +1342,17 @@ SDL_SYS_HapticSetAutocenter(SDL_Haptic * haptic, int autocenter)
     Uint32 val;
 
     /* Mac OS X only has 0 (off) and 1 (on) */
-    if (autocenter == 0)
+    if (autocenter == 0) {
         val = 0;
-    else
+    } else {
         val = 1;
+    }
 
     ret = FFDeviceSetForceFeedbackProperty(haptic->hwdata->device,
                                            FFPROP_AUTOCENTER, &val);
     if (ret != FF_OK) {
-        SDL_SetError("Haptic: Error setting autocenter: %s.",
-                     FFStrError(ret));
-        return -1;
+        return SDL_SetError("Haptic: Error setting autocenter: %s.",
+                            FFStrError(ret));
     }
 
     return 0;
@@ -1283,8 +1370,7 @@ SDL_SYS_HapticPause(SDL_Haptic * haptic)
     ret = FFDeviceSendForceFeedbackCommand(haptic->hwdata->device,
                                            FFSFFC_PAUSE);
     if (ret != FF_OK) {
-        SDL_SetError("Haptic: Error pausing device: %s.", FFStrError(ret));
-        return -1;
+        return SDL_SetError("Haptic: Error pausing device: %s.", FFStrError(ret));
     }
 
     return 0;
@@ -1302,8 +1388,7 @@ SDL_SYS_HapticUnpause(SDL_Haptic * haptic)
     ret = FFDeviceSendForceFeedbackCommand(haptic->hwdata->device,
                                            FFSFFC_CONTINUE);
     if (ret != FF_OK) {
-        SDL_SetError("Haptic: Error pausing device: %s.", FFStrError(ret));
-        return -1;
+        return SDL_SetError("Haptic: Error pausing device: %s.", FFStrError(ret));
     }
 
     return 0;
@@ -1321,12 +1406,12 @@ SDL_SYS_HapticStopAll(SDL_Haptic * haptic)
     ret = FFDeviceSendForceFeedbackCommand(haptic->hwdata->device,
                                            FFSFFC_STOPALL);
     if (ret != FF_OK) {
-        SDL_SetError("Haptic: Error stopping device: %s.", FFStrError(ret));
-        return -1;
+        return SDL_SetError("Haptic: Error stopping device: %s.", FFStrError(ret));
     }
 
     return 0;
 }
 
-
 #endif /* SDL_HAPTIC_IOKIT */
+
+/* vi: set ts=4 sw=4 expandtab: */
