@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2018 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,8 +20,8 @@
 */
 #include "../SDL_internal.h"
 
-#ifndef _SDL_thread_c_h
-#define _SDL_thread_c_h
+#ifndef SDL_thread_c_h_
+#define SDL_thread_c_h_
 
 #include "SDL_thread.h"
 
@@ -59,6 +59,7 @@ struct SDL_Thread
     SDL_atomic_t state;  /* SDL_THREAD_STATE_* */
     SDL_error errbuf;
     char *name;
+    size_t stacksize;  /* 0 for default, >0 for user-specified stack size. */
     void *data;
 };
 
@@ -70,7 +71,7 @@ typedef struct {
     unsigned int limit;
     struct {
         void *data;
-        void (*destructor)(void*);
+        void (SDLCALL *destructor)(void*);
     } array[1];
 } SDL_TLSData;
 
@@ -81,7 +82,7 @@ typedef struct {
    This is only intended as a fallback if getting real thread-local
    storage fails or isn't supported on this platform.
  */
-extern SDL_TLSData *SDL_Generic_GetTLSData();
+extern SDL_TLSData *SDL_Generic_GetTLSData(void);
 
 /* Set cross-platform, slow, thread local storage for this thread.
    This is only intended as a fallback if getting real thread-local
@@ -89,6 +90,6 @@ extern SDL_TLSData *SDL_Generic_GetTLSData();
  */
 extern int SDL_Generic_SetTLSData(SDL_TLSData *data);
 
-#endif /* _SDL_thread_c_h */
+#endif /* SDL_thread_c_h_ */
 
 /* vi: set ts=4 sw=4 expandtab: */
