@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2018 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -19,8 +19,8 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#ifndef _SDL_rpievents_c_h
-#define _SDL_rpievents_c_h
+#ifndef SDL_rpievents_c_h_
+#define SDL_rpievents_c_h_
 
 #include "SDL_rpivideo.h"
 
@@ -28,4 +28,4 @@ void RPI_PumpEvents(_THIS);
 void RPI_EventInit(_THIS);
 void RPI_EventQuit(_THIS);
 
-#endif /* _SDL_rpievents_c_h */
+#endif /* SDL_rpievents_c_h_ */
