@@ -1,5 +1,25 @@
+/*
+  Simple DirectMedia Layer
+  Copyright (C) 1997-2018 Sam Lantinga <slouken@libsdl.org>
 
-#define MAX_WMMSG	(sizeof(wmtab)/sizeof(wmtab[0]))
+  This software is provided 'as-is', without any express or implied
+  warranty.  In no event will the authors be held liable for any damages
+  arising from the use of this software.
+
+  Permission is granted to anyone to use this software for any purpose,
+  including commercial applications, and to alter it and redistribute it
+  freely, subject to the following restrictions:
+
+  1. The origin of this software must not be misrepresented; you must not
+     claim that you wrote the original software. If you use this software
+     in a product, an acknowledgment in the product documentation would be
+     appreciated but is not required.
+  2. Altered source versions must be plainly marked as such, and must not be
+     misrepresented as being the original software.
+  3. This notice may not be removed or altered from any source distribution.
+*/
+
+#define MAX_WMMSG   (sizeof(wmtab)/sizeof(wmtab[0]))
 
 char *wmtab[] = {
     "WM_NULL",
@@ -173,11 +193,11 @@ char *wmtab[] = {
     "WM_NCMBUTTONUP",
     "WM_NCMBUTTONDBLCLK",
     "UNKNOWN (170)",
-    "UNKNOWN (171)",
-    "UNKNOWN (172)",
-    "UNKNOWN (173)",
-    "UNKNOWN (174)",
-    "UNKNOWN (175)",
+    "WM_NCXBUTTONDOWN",
+    "WM_NCXBUTTONUP",
+    "WM_NCXBUTTONDBLCLK",
+    "WM_NCUAHDRAWCAPTION",
+    "WM_NCUAHDRAWFRAME",
     "UNKNOWN (176)",
     "UNKNOWN (177)",
     "UNKNOWN (178)",
@@ -257,7 +277,7 @@ char *wmtab[] = {
     "UNKNOWN (252)",
     "UNKNOWN (253)",
     "UNKNOWN (254)",
-    "UNKNOWN (255)",
+    "WM_INPUT",
     "WM_KEYDOWN",
     "WM_KEYUP",
     "WM_CHAR",
@@ -570,30 +590,30 @@ char *wmtab[] = {
     "UNKNOWN (565)",
     "UNKNOWN (566)",
     "UNKNOWN (567)",
-    "UNKNOWN (568)",
-    "UNKNOWN (569)",
-    "UNKNOWN (570)",
+    "WM_POINTERDEVICECHANGE",
+    "WM_POINTERDEVICEINRANGE",
+    "WM_POINTERDEVICEOUTOFRANGE",
     "UNKNOWN (571)",
     "UNKNOWN (572)",
     "UNKNOWN (573)",
     "UNKNOWN (574)",
     "UNKNOWN (575)",
     "WM_TOUCH",
-    "UNKNOWN (577)",
-    "UNKNOWN (578)",
-    "UNKNOWN (579)",
+    "WM_NCPOINTERUPDATE",
+    "WM_NCPOINTERDOWN",
+    "WM_NCPOINTERUP",
     "UNKNOWN (580)",
-    "UNKNOWN (581)",
-    "UNKNOWN (582)",
-    "UNKNOWN (583)",
-    "UNKNOWN (584)",
-    "UNKNOWN (585)",
-    "UNKNOWN (586)",
-    "UNKNOWN (587)",
-    "UNKNOWN (588)",
-    "UNKNOWN (589)",
-    "UNKNOWN (590)",
-    "UNKNOWN (591)",
+    "WM_POINTERUPDATE",
+    "WM_POINTERDOWN",
+    "WM_POINTERUP",
+    "WM_POINTERENTER",
+    "WM_POINTERLEAVE",
+    "WM_POINTERACTIVATE",
+    "WM_POINTERCAPTURECHANGED",
+    "WM_TOUCHHITTESTING",
+    "WM_POINTERWHEEL",
+    "WM_POINTERHWHEEL",
+    "DM_POINTERHITTEST",
     "UNKNOWN (592)",
     "UNKNOWN (593)",
     "UNKNOWN (594)",
@@ -643,14 +663,14 @@ char *wmtab[] = {
     "UNKNOWN (638)",
     "UNKNOWN (639)",
     "UNKNOWN (640)",
-    "UNKNOWN (641)",
-    "UNKNOWN (642)",
-    "UNKNOWN (643)",
-    "UNKNOWN (644)",
-    "UNKNOWN (645)",
-    "UNKNOWN (646)",
+    "WM_IME_SETCONTEXT",
+    "WM_IME_NOTIFY",
+    "WM_IME_CONTROL",
+    "WM_IME_COMPOSITIONFULL",
+    "WM_IME_SELECT",
+    "WM_IME_CHAR",
     "UNKNOWN (647)",
-    "UNKNOWN (648)",
+    "WM_IME_REQUEST",
     "UNKNOWN (649)",
     "UNKNOWN (650)",
     "UNKNOWN (651)",
@@ -658,8 +678,8 @@ char *wmtab[] = {
     "UNKNOWN (653)",
     "UNKNOWN (654)",
     "UNKNOWN (655)",
-    "UNKNOWN (656)",
-    "UNKNOWN (657)",
+    "WM_IME_KEYDOWN",
+    "WM_IME_KEYUP",
     "UNKNOWN (658)",
     "UNKNOWN (659)",
     "UNKNOWN (660)",
@@ -674,9 +694,9 @@ char *wmtab[] = {
     "UNKNOWN (669)",
     "UNKNOWN (670)",
     "UNKNOWN (671)",
-    "UNKNOWN (672)",
+    "WM_NCMOUSEHOVER",
     "WM_MOUSEHOVER",
-    "UNKNOWN (674)",
+    "WM_NCMOUSELEAVE",
     "WM_MOUSELEAVE",
     "UNKNOWN (676)",
     "UNKNOWN (677)",
@@ -691,7 +711,7 @@ char *wmtab[] = {
     "UNKNOWN (686)",
     "UNKNOWN (687)",
     "UNKNOWN (688)",
-    "UNKNOWN (689)",
+    "WM_WTSSESSION_CHANGE",
     "UNKNOWN (690)",
     "UNKNOWN (691)",
     "UNKNOWN (692)",
@@ -738,7 +758,7 @@ char *wmtab[] = {
     "UNKNOWN (733)",
     "UNKNOWN (734)",
     "UNKNOWN (735)",
-    "UNKNOWN (736)",
+    "WM_DPICHANGED",
     "UNKNOWN (737)",
     "UNKNOWN (738)",
     "UNKNOWN (739)",
@@ -795,20 +815,20 @@ char *wmtab[] = {
     "UNKNOWN (790)",
     "WM_PRINT",
     "WM_PRINTCLIENT",
-    "UNKNOWN (793)",
-    "UNKNOWN (794)",
+    "WM_APPCOMMAND",
+    "WM_THEMECHANGED",
     "UNKNOWN (795)",
     "UNKNOWN (796)",
-    "UNKNOWN (797)",
-    "UNKNOWN (798)",
-    "UNKNOWN (799)",
-    "UNKNOWN (800)",
-    "UNKNOWN (801)",
+    "WM_CLIPBOARDUPDATE",
+    "WM_DWMCOMPOSITIONCHANGED",
+    "WM_DWMNCRENDERINGCHANGED",
+    "WM_DWMCOLORIZATIONCOLORCHANGED",
+    "WM_DWMWINDOWMAXIMIZEDCHANGE",
     "UNKNOWN (802)",
-    "UNKNOWN (803)",
+    "WM_DWMSENDICONICTHUMBNAIL",
     "UNKNOWN (804)",
     "UNKNOWN (805)",
-    "UNKNOWN (806)",
+    "WM_DWMSENDICONICLIVEPREVIEWBITMAP",
     "UNKNOWN (807)",
     "UNKNOWN (808)",
     "UNKNOWN (809)",
@@ -833,7 +853,7 @@ char *wmtab[] = {
     "UNKNOWN (828)",
     "UNKNOWN (829)",
     "UNKNOWN (830)",
-    "UNKNOWN (831)",
+    "WM_GETTITLEBARINFOEX",
     "UNKNOWN (832)",
     "UNKNOWN (833)",
     "UNKNOWN (834)",
