@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2011 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2018 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,30 +18,15 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "SDL_config.h"
+#include "../../SDL_internal.h"
 
-#ifndef _SDL_x11touch_h
-#define _SDL_x11touch_h
-
-/*
-!!! FIXME: remove the evdev code;
-!!! FIXME: this should be using XInput 2.1's new multitouch API.
-*/
-
-/* What should this be? */
-#ifdef SDL_INPUT_LINUXEV
-typedef struct EventTouchData
-{
-    int x,y,pressure,finger; /* Temporary Variables until sync */
-    int eventStream;
-    SDL_bool up;
-    SDL_bool down;
-} EventTouchData;
-#endif
+#ifndef SDL_x11touch_h_
+#define SDL_x11touch_h_
 
 extern void X11_InitTouch(_THIS);
 extern void X11_QuitTouch(_THIS);
+extern void X11_ResetTouch(_THIS);
 
-#endif /* _SDL_x11touch_h */
+#endif /* SDL_x11touch_h_ */
 
 /* vi: set ts=4 sw=4 expandtab: */
