@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997-2011 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2018 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -12,9 +12,6 @@
 #include <stdio.h>
 
 #include "SDL.h"
-#include "SDL_atomic.h"
-#include "SDL_assert.h"
-#include "SDL_cpuinfo.h"
 
 /*
   Absolutely basic tests just to see if we get the expected value
@@ -45,39 +42,39 @@ void RunBasicTest()
     SDL_atomic_t v;
     SDL_bool tfret = SDL_FALSE;
 
-    printf("\nspin lock---------------------------------------\n\n");
+    SDL_Log("\nspin lock---------------------------------------\n\n");
 
     SDL_AtomicLock(&lock);
-    printf("AtomicLock                   lock=%d\n", lock);
+    SDL_Log("AtomicLock                   lock=%d\n", lock);
     SDL_AtomicUnlock(&lock);
-    printf("AtomicUnlock                 lock=%d\n", lock);
+    SDL_Log("AtomicUnlock                 lock=%d\n", lock);
+
+    SDL_Log("\natomic -----------------------------------------\n\n");
 
-    printf("\natomic -----------------------------------------\n\n");
-     
     SDL_AtomicSet(&v, 0);
-    tfret = SDL_AtomicSet(&v, 10) == 0;
-    printf("AtomicSet(10)        tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
-    tfret = SDL_AtomicAdd(&v, 10) == 10;
-    printf("AtomicAdd(10)        tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
+    tfret = SDL_AtomicSet(&v, 10) == 0 ? SDL_TRUE : SDL_FALSE;
+    SDL_Log("AtomicSet(10)        tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
+    tfret = SDL_AtomicAdd(&v, 10) == 10 ? SDL_TRUE : SDL_FALSE;
+    SDL_Log("AtomicAdd(10)        tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
 
     SDL_AtomicSet(&v, 0);
     SDL_AtomicIncRef(&v);
-    tfret = (SDL_AtomicGet(&v) == 1);
-    printf("AtomicIncRef()       tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
+    tfret = (SDL_AtomicGet(&v) == 1) ? SDL_TRUE : SDL_FALSE;
+    SDL_Log("AtomicIncRef()       tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
     SDL_AtomicIncRef(&v);
-    tfret = (SDL_AtomicGet(&v) == 2);
-    printf("AtomicIncRef()       tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
-    tfret = (SDL_AtomicDecRef(&v) == SDL_FALSE);
-    printf("AtomicDecRef()       tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
-    tfret = (SDL_AtomicDecRef(&v) == SDL_TRUE);
-    printf("AtomicDecRef()       tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
+    tfret = (SDL_AtomicGet(&v) == 2) ? SDL_TRUE : SDL_FALSE;
+    SDL_Log("AtomicIncRef()       tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
+    tfret = (SDL_AtomicDecRef(&v) == SDL_FALSE) ? SDL_TRUE : SDL_FALSE;
+    SDL_Log("AtomicDecRef()       tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
+    tfret = (SDL_AtomicDecRef(&v) == SDL_TRUE) ? SDL_TRUE : SDL_FALSE;
+    SDL_Log("AtomicDecRef()       tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
 
     SDL_AtomicSet(&v, 10);
-    tfret = (SDL_AtomicCAS(&v, 0, 20) == SDL_FALSE);
-    printf("AtomicCAS()          tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
+    tfret = (SDL_AtomicCAS(&v, 0, 20) == SDL_FALSE) ? SDL_TRUE : SDL_FALSE;
+    SDL_Log("AtomicCAS()          tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
     value = SDL_AtomicGet(&v);
-    tfret = (SDL_AtomicCAS(&v, value, 20) == SDL_TRUE);
-    printf("AtomicCAS()          tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
+    tfret = (SDL_AtomicCAS(&v, value, 20) == SDL_TRUE) ? SDL_TRUE : SDL_FALSE;
+    SDL_Log("AtomicCAS()          tfret=%s val=%d\n", tf(tfret), SDL_AtomicGet(&v));
 }
 
 /**************************************************************************/
@@ -91,36 +88,39 @@ void RunBasicTest()
 /* Tests semantics of atomic operations.  Also a stress test
  * to see if they are really atomic.
  *
- * Serveral threads adding to the same variable.
+ * Several threads adding to the same variable.
  * at the end the value is compared with the expected
  * and with a non-atomic counter.
  */
- 
+
 /* Number of concurrent incrementers */
 #define NThreads 2
 #define CountInc 100
 #define VALBITS (sizeof(atomicValue)*8)
- 
+
 #define atomicValue int
 #define CountTo ((atomicValue)((unsigned int)(1<<(VALBITS-1))-1))
 #define NInter (CountTo/CountInc/NThreads)
 #define Expect (CountTo-NInter*CountInc*NThreads)
- 
-SDL_COMPILE_TIME_ASSERT(size, CountTo>0); /* check for rollover */
- 
+
+enum {
+   CountTo_GreaterThanZero = CountTo > 0,
+};
+SDL_COMPILE_TIME_ASSERT(size, CountTo_GreaterThanZero); /* check for rollover */
+
 static SDL_atomic_t good = { 42 };
- 
+
 static atomicValue bad = 42;
- 
+
 static SDL_atomic_t threadsRunning;
 
 static SDL_sem *threadDone;
- 
+
 static
-int adder(void* junk)
+int SDLCALL adder(void* junk)
 {
     unsigned long N=NInter;
-    printf("Thread subtracting %d %lu times\n",CountInc,N);
+    SDL_Log("Thread subtracting %d %lu times\n",CountInc,N);
     while (N--) {
         SDL_AtomicAdd(&good, -CountInc);
         bad-=CountInc;
@@ -129,113 +129,113 @@ int adder(void* junk)
     SDL_SemPost(threadDone);
     return 0;
 }
- 
+
 static
 void runAdder(void)
 {
     Uint32 start, end;
     int T=NThreads;
- 
+
     start = SDL_GetTicks();
- 
+
     threadDone = SDL_CreateSemaphore(0);
 
     SDL_AtomicSet(&threadsRunning, NThreads);
 
     while (T--)
         SDL_CreateThread(adder, "Adder", NULL);
- 
+
     while (SDL_AtomicGet(&threadsRunning) > 0)
         SDL_SemWait(threadDone);
- 
+
     SDL_DestroySemaphore(threadDone);
 
     end = SDL_GetTicks();
- 
-    printf("Finished in %f sec\n", (end - start) / 1000.f);
+
+    SDL_Log("Finished in %f sec\n", (end - start) / 1000.f);
 }
- 
+
 static
 void RunEpicTest()
 {
     int b;
     atomicValue v;
- 
-    printf("\nepic test---------------------------------------\n\n");
 
-    printf("Size asserted to be >= 32-bit\n");
+    SDL_Log("\nepic test---------------------------------------\n\n");
+
+    SDL_Log("Size asserted to be >= 32-bit\n");
     SDL_assert(sizeof(atomicValue)>=4);
- 
-    printf("Check static initializer\n");
+
+    SDL_Log("Check static initializer\n");
     v=SDL_AtomicGet(&good);
     SDL_assert(v==42);
- 
+
     SDL_assert(bad==42);
- 
-    printf("Test negative values\n");
+
+    SDL_Log("Test negative values\n");
     SDL_AtomicSet(&good, -5);
     v=SDL_AtomicGet(&good);
     SDL_assert(v==-5);
- 
-    printf("Verify maximum value\n");
+
+    SDL_Log("Verify maximum value\n");
     SDL_AtomicSet(&good, CountTo);
     v=SDL_AtomicGet(&good);
     SDL_assert(v==CountTo);
- 
-    printf("Test compare and exchange\n");
- 
+
+    SDL_Log("Test compare and exchange\n");
+
     b=SDL_AtomicCAS(&good, 500, 43);
     SDL_assert(!b); /* no swap since CountTo!=500 */
     v=SDL_AtomicGet(&good);
     SDL_assert(v==CountTo); /* ensure no swap */
- 
+
     b=SDL_AtomicCAS(&good, CountTo, 44);
     SDL_assert(!!b); /* will swap */
     v=SDL_AtomicGet(&good);
     SDL_assert(v==44);
- 
-    printf("Test Add\n");
- 
+
+    SDL_Log("Test Add\n");
+
     v=SDL_AtomicAdd(&good, 1);
     SDL_assert(v==44);
     v=SDL_AtomicGet(&good);
     SDL_assert(v==45);
- 
+
     v=SDL_AtomicAdd(&good, 10);
     SDL_assert(v==45);
     v=SDL_AtomicGet(&good);
     SDL_assert(v==55);
- 
-    printf("Test Add (Negative values)\n");
- 
+
+    SDL_Log("Test Add (Negative values)\n");
+
     v=SDL_AtomicAdd(&good, -20);
     SDL_assert(v==55);
     v=SDL_AtomicGet(&good);
     SDL_assert(v==35);
- 
+
     v=SDL_AtomicAdd(&good, -50); /* crossing zero down */
     SDL_assert(v==35);
     v=SDL_AtomicGet(&good);
     SDL_assert(v==-15);
- 
+
     v=SDL_AtomicAdd(&good, 30); /* crossing zero up */
     SDL_assert(v==-15);
     v=SDL_AtomicGet(&good);
     SDL_assert(v==15);
- 
-    printf("Reset before count down test\n");
+
+    SDL_Log("Reset before count down test\n");
     SDL_AtomicSet(&good, CountTo);
     v=SDL_AtomicGet(&good);
     SDL_assert(v==CountTo);
- 
+
     bad=CountTo;
     SDL_assert(bad==CountTo);
- 
-    printf("Counting down from %d, Expect %d remaining\n",CountTo,Expect);
+
+    SDL_Log("Counting down from %d, Expect %d remaining\n",CountTo,Expect);
     runAdder();
- 
+
     v=SDL_AtomicGet(&good);
-    printf("Atomic %d Non-Atomic %d\n",v,bad);
+    SDL_Log("Atomic %d Non-Atomic %d\n",v,bad);
     SDL_assert(v==Expect);
     SDL_assert(bad!=Expect);
 }
@@ -287,7 +287,7 @@ typedef struct
     char cache_pad4[SDL_CACHELINE_SIZE-sizeof(SDL_SpinLock)-2*sizeof(SDL_atomic_t)];
 #endif
 
-    volatile SDL_bool active;
+    SDL_atomic_t active;
 
     /* Only needed for the mutex test */
     SDL_mutex *mutex;
@@ -306,8 +306,9 @@ static void InitEventQueue(SDL_EventQueue *queue)
 #ifdef TEST_SPINLOCK_FIFO
     queue->lock = 0;
     SDL_AtomicSet(&queue->rwcount, 0);
+    SDL_AtomicSet(&queue->watcher, 0);
 #endif
-    queue->active = SDL_TRUE;
+    SDL_AtomicSet(&queue->active, 1);
 }
 
 static SDL_bool EnqueueEvent_LockFree(SDL_EventQueue *queue, const SDL_Event *event)
@@ -412,7 +413,7 @@ static SDL_bool EnqueueEvent_Mutex(SDL_EventQueue *queue, const SDL_Event *event
     int delta;
     SDL_bool status = SDL_FALSE;
 
-    SDL_mutexP(queue->mutex);
+    SDL_LockMutex(queue->mutex);
 
     queue_pos = (unsigned)queue->enqueue_pos.value;
     entry = &queue->entries[queue_pos & WRAP_MASK];
@@ -429,10 +430,10 @@ static SDL_bool EnqueueEvent_Mutex(SDL_EventQueue *queue, const SDL_Event *event
     } else if (delta < 0) {
         /* We ran into an old queue entry, which means it still needs to be dequeued */
     } else {
-        printf("ERROR: mutex failed!\n");
+        SDL_Log("ERROR: mutex failed!\n");
     }
 
-    SDL_mutexV(queue->mutex);
+    SDL_UnlockMutex(queue->mutex);
 
     return status;
 }
@@ -445,7 +446,7 @@ static SDL_bool DequeueEvent_Mutex(SDL_EventQueue *queue, SDL_Event *event)
     int delta;
     SDL_bool status = SDL_FALSE;
 
-    SDL_mutexP(queue->mutex);
+    SDL_LockMutex(queue->mutex);
 
     queue_pos = (unsigned)queue->dequeue_pos.value;
     entry = &queue->entries[queue_pos & WRAP_MASK];
@@ -462,10 +463,10 @@ static SDL_bool DequeueEvent_Mutex(SDL_EventQueue *queue, SDL_Event *event)
     } else if (delta < 0) {
         /* We ran into an old queue entry, which means we've hit empty */
     } else {
-        printf("ERROR: mutex failed!\n");
+        SDL_Log("ERROR: mutex failed!\n");
     }
 
-    SDL_mutexV(queue->mutex);
+    SDL_UnlockMutex(queue->mutex);
 
     return status;
 }
@@ -494,11 +495,10 @@ typedef struct
     char padding[SDL_CACHELINE_SIZE-(sizeof(SDL_EventQueue*)+sizeof(int)*NUM_WRITERS+sizeof(int)+sizeof(SDL_bool))%SDL_CACHELINE_SIZE];
 } ReaderData;
 
-static int FIFO_Writer(void* _data)
+static int SDLCALL FIFO_Writer(void* _data)
 {
     WriterData *data = (WriterData *)_data;
     SDL_EventQueue *queue = data->queue;
-    int index = data->index;
     int i;
     SDL_Event event;
 
@@ -530,7 +530,7 @@ static int FIFO_Writer(void* _data)
     return 0;
 }
 
-static int FIFO_Reader(void* _data)
+static int SDLCALL FIFO_Reader(void* _data)
 {
     ReaderData *data = (ReaderData *)_data;
     SDL_EventQueue *queue = data->queue;
@@ -541,7 +541,7 @@ static int FIFO_Reader(void* _data)
             if (DequeueEvent_LockFree(queue, &event)) {
                 WriterData *writer = (WriterData*)event.user.data1;
                 ++data->counters[writer->index];
-            } else if (queue->active) {
+            } else if (SDL_AtomicGet(&queue->active)) {
                 ++data->waits;
                 SDL_Delay(0);
             } else {
@@ -554,7 +554,7 @@ static int FIFO_Reader(void* _data)
             if (DequeueEvent_Mutex(queue, &event)) {
                 WriterData *writer = (WriterData*)event.user.data1;
                 ++data->counters[writer->index];
-            } else if (queue->active) {
+            } else if (SDL_AtomicGet(&queue->active)) {
                 ++data->waits;
                 SDL_Delay(0);
             } else {
@@ -570,11 +570,11 @@ static int FIFO_Reader(void* _data)
 
 #ifdef TEST_SPINLOCK_FIFO
 /* This thread periodically locks the queue for no particular reason */
-static int FIFO_Watcher(void* _data)
+static int SDLCALL FIFO_Watcher(void* _data)
 {
     SDL_EventQueue *queue = (SDL_EventQueue *)_data;
 
-    while (queue->active) {
+    while (SDL_AtomicGet(&queue->active)) {
         SDL_AtomicLock(&queue->lock);
         SDL_AtomicIncRef(&queue->watcher);
         while (SDL_AtomicGet(&queue->rwcount) > 0) {
@@ -599,9 +599,11 @@ static void RunFIFOTest(SDL_bool lock_free)
     Uint32 start, end;
     int i, j;
     int grand_total;
- 
-    printf("\nFIFO test---------------------------------------\n\n");
-    printf("Mode: %s\n", lock_free ? "LockFree" : "Mutex");
+    char textBuffer[1024];
+    size_t len;
+
+    SDL_Log("\nFIFO test---------------------------------------\n\n");
+    SDL_Log("Mode: %s\n", lock_free ? "LockFree" : "Mutex");
 
     readersDone = SDL_CreateSemaphore(0);
     writersDone = SDL_CreateSemaphore(0);
@@ -614,7 +616,7 @@ static void RunFIFOTest(SDL_bool lock_free)
     }
 
     start = SDL_GetTicks();
- 
+
 #ifdef TEST_SPINLOCK_FIFO
     /* Start a monitoring thread */
     if (lock_free) {
@@ -623,7 +625,7 @@ static void RunFIFOTest(SDL_bool lock_free)
 #endif
 
     /* Start the readers first */
-    printf("Starting %d readers\n", NUM_READERS);
+    SDL_Log("Starting %d readers\n", NUM_READERS);
     SDL_zero(readerData);
     SDL_AtomicSet(&readersRunning, NUM_READERS);
     for (i = 0; i < NUM_READERS; ++i) {
@@ -635,7 +637,7 @@ static void RunFIFOTest(SDL_bool lock_free)
     }
 
     /* Start up the writers */
-    printf("Starting %d writers\n", NUM_WRITERS);
+    SDL_Log("Starting %d writers\n", NUM_WRITERS);
     SDL_zero(writerData);
     SDL_AtomicSet(&writersRunning, NUM_WRITERS);
     for (i = 0; i < NUM_WRITERS; ++i) {
@@ -646,14 +648,14 @@ static void RunFIFOTest(SDL_bool lock_free)
         writerData[i].lock_free = lock_free;
         SDL_CreateThread(FIFO_Writer, name, &writerData[i]);
     }
- 
+
     /* Wait for the writers */
     while (SDL_AtomicGet(&writersRunning) > 0) {
         SDL_SemWait(writersDone);
     }
- 
+
     /* Shut down the queue so readers exit */
-    queue.active = SDL_FALSE;
+    SDL_AtomicSet(&queue.active, 0);
 
     /* Wait for the readers */
     while (SDL_AtomicGet(&readersRunning) > 0) {
@@ -661,24 +663,24 @@ static void RunFIFOTest(SDL_bool lock_free)
     }
 
     end = SDL_GetTicks();
- 
+
     SDL_DestroySemaphore(readersDone);
     SDL_DestroySemaphore(writersDone);
 
     if (!lock_free) {
         SDL_DestroyMutex(queue.mutex);
     }
- 
-    printf("Finished in %f sec\n", (end - start) / 1000.f);
 
-    printf("\n");
+    SDL_Log("Finished in %f sec\n", (end - start) / 1000.f);
+
+    SDL_Log("\n");
     for (i = 0; i < NUM_WRITERS; ++i) {
-        printf("Writer %d wrote %d events, had %d waits\n", i, EVENTS_PER_WRITER, writerData[i].waits);
+        SDL_Log("Writer %d wrote %d events, had %d waits\n", i, EVENTS_PER_WRITER, writerData[i].waits);
     }
-    printf("Writers wrote %d total events\n", NUM_WRITERS*EVENTS_PER_WRITER);
+    SDL_Log("Writers wrote %d total events\n", NUM_WRITERS*EVENTS_PER_WRITER);
 
     /* Print a breakdown of which readers read messages from which writer */
-    printf("\n");
+    SDL_Log("\n");
     grand_total = 0;
     for (i = 0; i < NUM_READERS; ++i) {
         int total = 0;
@@ -686,17 +688,21 @@ static void RunFIFOTest(SDL_bool lock_free)
             total += readerData[i].counters[j];
         }
         grand_total += total;
-        printf("Reader %d read %d events, had %d waits\n", i, total, readerData[i].waits);
-        printf("  { ");
+        SDL_Log("Reader %d read %d events, had %d waits\n", i, total, readerData[i].waits);
+        SDL_snprintf(textBuffer, sizeof(textBuffer), "  { ");
         for (j = 0; j < NUM_WRITERS; ++j) {
             if (j > 0) {
-                printf(", ");
+                len = SDL_strlen(textBuffer);
+                SDL_snprintf(textBuffer + len, sizeof(textBuffer) - len, ", ");
             }
-            printf("%d", readerData[i].counters[j]);
+            len = SDL_strlen(textBuffer);
+            SDL_snprintf(textBuffer + len, sizeof(textBuffer) - len, "%d", readerData[i].counters[j]);
         }
-        printf(" }\n");
+        len = SDL_strlen(textBuffer);
+        SDL_snprintf(textBuffer + len, sizeof(textBuffer) - len, " }\n");
+        SDL_Log("%s", textBuffer);
     }
-    printf("Readers read %d total events\n", grand_total);
+    SDL_Log("Readers read %d total events\n", grand_total);
 }
 
 /* End FIFO test */
@@ -705,6 +711,9 @@ static void RunFIFOTest(SDL_bool lock_free)
 int
 main(int argc, char *argv[])
 {
+    /* Enable standard application logging */
+    SDL_LogSetPriority(SDL_LOG_CATEGORY_APPLICATION, SDL_LOG_PRIORITY_INFO);
+
     RunBasicTest();
     RunEpicTest();
 /* This test is really slow, so don't run it by default */
